﻿using GpsCardGatewayPosition.Common;

namespace GpsCardGatewayPosition.Service.Biz.Location.Dto.Wayz
{
    public class WayzRequest
    {
    

        public long Timestamp { get; set; } = Utils.ConvertToSTimeStamp(DateTime.Now);

        //	Id = "679e9043-14fe-44df-aba6-b2fd62592fea",
        //	Timestamp = 1670231477000,
        //	Asset = new AssetDevice
        //	{
        //		Id = "679e9043-14fe-44df-aba6-b2fd62592fea",
        //		Manufacturer = "xiaomi",
        //		ImeiMd5 = "1be14c6210b3115f",
        //		UniqueId = "8f95204d-a60e-40bb-b8be-45a1ab3bef2a"
        //	},
        public virtual string Id { get; set; } = Guid.NewGuid().ToString("D");

        public virtual AssetDevice Asset { get; set; } = new AssetDevice
        {
            Id= "679e9043-14fe-44df-aba6-b2fd62592fea",
            Manufacturer= "telpo",
            ImeiMd5= "1be14c6210b3115f",
            UniqueId= "8f95204d-a60e-40bb-b8be-45a1ab3bef2a"

        }; 

        public LocationDetail Location { get; set; }

        public class LocationDetail
        {
            /// <summary>
            /// Wifi
            /// </summary>
            public List<WifiItem> Wifis { get; set; }

            /// <summary>
            /// LBS/Cellular 基站
            /// </summary>
            public List<CellularItem> Cellulars { get; set; }
            /// <summary>
            /// Gnss 地球系坐标
            /// </summary>
            public Gnss Gnss { get; set; }

            /// <summary>
            ///  gcj02 火星系坐标
            /// </summary>
            public Postion Position { get; set; }

        }

        // public List<WifiItem> Wifis { get; set; }

        public class AssetDevice
        {
            public string Id { get; set; }
            public string Manufacturer { get; set; }
            public string ImeiMd5 { get; set; }
            public string UniqueId { get; set; }

        }

        public class Item 
        {
            /// <summary>
            /// 扫描到的时间戳（UTC 时间）
            /// </summary>
            public long Timestamp { get; set; }= Utils.ConvertToSTimeStamp(DateTime.Now);
            /// <summary>
            /// 设备是否已连接在该 AP 下，如果是"False"则可以省略此 字段
            /// </summary>
            public bool Connected { get; set; } = default;
            /// <summary>
            /// 信号强度，单位为：-dBm，即取真实信号强度的 绝 对值
            /// </summary>
            public virtual int SignalStrength { get; set; } = default;
        }
        public class WifiItem: Item
        {
            /// <summary>
            /// 设备网卡的 MAC 地址
            /// </summary>
            public string MacAddress { get; set; }
            /// <summary>
            /// WiFi 的ssid 名称，注意：JSON 编码时需要进行转义编码
            /// </summary>
            public string Ssid { get; set; } = default;
            /// <summary>
            /// 信号强度，单位为：-dBm，即取真实信号强度的绝对值
            /// </summary>
            public override int SignalStrength { get; set; }
            /// <summary>
            /// 指定通道下的频率，单位： MHz
            /// </summary>
            public int Frequency { get; set; } = default;
            /// <summary>
            /// WiFi 子频段的通道编号
            /// </summary>
            public int Channel { get; set; } = default;

            
        }
        public class CellularItem: Item
        {
            /// <summary>
            /// 小区 ID，当 CDMA 时，为 BID（Base Station ID）
            /// </summary>
            public int CellId { get; set; }
            /// <summary>
            /// 基站类 型， 只能 是以 下 值：gsm, wcdma, lte, cdma
            /// </summary>
            public string RadioType { get; set; }
            /// <summary>
            /// MCC 码
            /// </summary>
            public int MobileCountryCode { get; set; }
            /// <summary>
            /// 当 CDMA 时，为 SID（System ID）码
            /// </summary>
            public int MobileNetworkCode { get; set; }
            /// <summary>
            /// 当 CDMA 时，为 NID（Network ID）； 
            /// 当 LTE 时，为 TAC（Tracking Area code）
            /// </summary>
            public int LocationAreaCode { get; set; }

        }

        public class Gnss 
        {
            /// <summary>
            /// 数据收集的时间戳（UTC 时间，单位：毫秒）
            /// </summary>
            public long Timestamp { get; set; }

            public Point Point { get; set; }

            /// <summary>
            /// 卫星定位水平精度，单位：米
            /// </summary>
            public float Accuracy { get; set; } = 0f;
            /// <summary>
            /// 卫星定位垂直精度，单位：米
            /// </summary>
            public float VerticalAccuracy { get; set; } = default;
            /// <summary>
            /// 速度，单位：米/秒
            /// </summary>
            public float Velocity { get; set; }
            /// <summary>
            /// 卫星定位垂直精度，单位：米
            /// </summary>
            public float VelocityAccuracy { get; set; } = default;
            /// <summary>
            /// 方向，单位：度，从北开始，顺时针 0~360 度
            /// </summary>
            public int Heading { get; set; } = 0;
        }

        public class Postion 
        {
            public long Timestamp { get; set; }

            public Point Point { get; set; }

            public float Accuracy { get; set; } = 0f;

            public string SpatialReference { get; set; } = "gcj02";

        }
        public class Point
        {
            /// <summary>
            /// 经度
            /// </summary>
            // public double Longitude { get; set; } // 维智精度
            public decimal Longitude { get; set; } // 高德精度

            /// <summary>
            /// 纬度
            /// </summary>
            // public double Latitude { get; set; } // 维智精度
            public decimal Latitude { get; set; } // 高德精度
            /// <summary>
            /// 卫星定位水平精度，单位：米
            /// </summary>
            public float Altitude { get; set; }=default;
        }
    }


}
